(require 'cl)

;; load ELI first
;(load "fi-site-init")


:;;; required configuration:

(setf fi:start-lisp-interface-arguments
      (function
        (lambda (use-background-streams-p core-file)
	  (list "--core" core-file
		"--eval" "(require :lep)"
		"--eval" "(lep:start-emacs-lisp-interface)"))))

;;; Sent to the a newly started listener.
(defun fi::setup-tcp-connection (proc)
  (format "%d\n" (fi::tcp-listener-generation proc)))

(defun fi::install-mode-line-run-status ()
  (setq mode-line-buffer-identification
    '("SBCL " fi:allegro-run-status-string " %12b")))


;;;; sample invocation:

(defun sbcl ()
  (interactive)
  (fi:common-lisp buffer-name
		  "/home/david/src/"
		  "/home/david/usr/sbcl-mt/bin/sbcl"
		  nil
		  "localhost"
		  "/home/david/usr/sbcl-mt/lib/sbcl/sbcl.core"))

(defun connect-to-sbcl ()
  (interactive)
  (let ((buffer-name "*sbcl*"))
    (fi:common-lisp buffer-name
		    "/home/david/src/"
		    "/home/david/src/lep/listener"
		    nil
		    "localhost"
		    nil)
    (let ((process (get-buffer-process buffer-name)))
      (push (cons process (- (process-id process))) fi::tcp-listener-table))))


;;;; David's private changes:

(defun eval-cl-expression ()
  (interactive)
  (fi:eval-in-lisp-asynchronous
   "(lep::eval-in-emacs (format nil \"(message ~S)\" (format nil \"~S\" %s)))"
   (read-from-minibuffer "cl:eval: ")))

(define-key global-map [?\C-c ?:] 'eval-cl-expression)

(defun fix-eli-keymaps ()
  (macrolet ((define-listener-key (key fn)
               `(progn
		  (define-key fi:inferior-common-lisp-mode-map ,key ,fn)
		  (define-key fi:lisp-listener-mode-map ,key ,fn))))
    (define-listener-key [(control m)]
      (lambda ()
	"If at the end of the buffer, insert a newline of send the command, as
  appropriate.  Else, go the the end of the buffer."
	;; ...instead of sending random garbage, as ELI does by default...!!!1!
	(interactive)
	(if (= (point) (point-max))
	    (fi:inferior-lisp-newline)
	  (goto-char (point-max)))))
    ;; C-c C-p C-c C-p C-c C-p C-c C-p C-c Argh
    (define-listener-key [(meta p)] 'fi:pop-input)
    (define-listener-key [(meta n)] 'fi:push-input)
    (define-listener-key [(control a)]
      (lambda (arg)
	"Move to the beginning of the input line, right _after_ the prompt.
         With a prefix argument, move to the beginning of the line proper."
	(interactive "p")
	(if (= 4 arg)
	    (beginning-of-line)
	  (fi:subprocess-beginning-of-line nil))))))
