(in-package :lep)

;;;; sb-threads don't have readable names...

(defclass thread ()
    ((pid :initarg :pid :accessor pid)
     (name :initarg :name :accessor name)))

(defvar *threads* (make-hash-table))

(defun get-thread (&optional (pid (sb-thread:current-thread-id)))
  (gethash pid *threads*))

(defun make-thread (name function &rest arguments)
  (let ((lock (sb-thread:make-mutex)))
    (labels ((make-object (pid)
	       (sb-thread:with-mutex (lock)
		 (cond
		   ((gethash pid *threads*))
		   (t (setf (gethash pid *threads*)
			    (make-instance 'thread :pid pid :name name))))))
	     (doit ()
	       (make-object (sb-thread:current-thread-id))
	       (unwind-protect
		   (apply function arguments)
		 (remhash (sb-thread:current-thread-id) *threads*))))
      (make-object (sb-thread:make-thread #'doit)))))

;; stolen from sb-aclrepl; would be nice if sb-thread provided it
(defun thread-pids ()
  "Return a list of the pids for all threads"
  (let ((offset (* 4 sb-vm::thread-pid-slot)))
    (sb-thread::mapcar-threads
     #'(lambda (sap) (sb-sys:sap-ref-32 sap offset)))))

(defun show-processes (&optional (stream t))
  (let ((pids (thread-pids))
	(current-pid (sb-thread:current-thread-id)))
    (format t "~&PID ~10TName~%~79,,,'-<~>~%")
    (dolist (pid pids)
      (format stream "~&~D" pid)
      (awhen (get-thread pid)
	(format stream " ~10T~D" (name %)))
      (when (= pid current-pid)
	(format stream "~60T[current listener]")))))

(eval-when (:compile-toplevel :load-toplevel :execute)
  (when (find-package :sb-aclrepl)
    (pushnew :lep-sb-aclrepl *features*)))

#+lep-sb-aclrepl
(sb-aclrepl:alias "processes" ()
  (show-processes sb-aclrepl::*output*))

#+lep-sb-aclrepl
(sb-aclrepl:alias "pro" ()
  (show-processes sb-aclrepl::*output*))
